package unibo.cs.labss.editor.il;

import java.io.PrintStream;

import unibo.cs.labss.editor.AbstractEditorKit;
import unibo.cs.labss.editor.IActionFactory;
import unibo.cs.labss.editor.IExamplesFactory;
import unibo.cs.labss.editor.IPartFactory;
import unibo.cs.labss.editor.IPrototypeManager;
import unibo.cs.labss.editor.il.actions.ILActionFactory;
import unibo.cs.labss.editor.il.actions.ILCommandFactory;
import unibo.cs.labss.editor.il.model.IILEntity;
import unibo.cs.labss.editor.il.model.impl.ILExamplesFactory;
import unibo.cs.labss.editor.il.model.impl.ILPrototypeManager;
import unibo.cs.labss.editor.il.model.visitors.InterpreterVisitor;
import unibo.cs.labss.editor.il.model.visitors.PrettyPrinterVisitor;
import unibo.cs.labss.editor.il.parts.ILPartFactoryVisitor;
import unibo.cs.labss.editor.model.IEntity;
import unibo.cs.labss.editor.requests.ICommandFactory;

/**
 * @author Riccardo Solmi
 */
public class ILEditorKit extends AbstractEditorKit {
	public static final String ID = "unibo.cs.labss.editor.il.ILEditorKit";

	public String getId() {
		return ID;
	}
	public String getName() {
		return "IL";
	}

	public IPrototypeManager getPrototypeManager() {
		return ILPrototypeManager.instance();
	}

	public IPartFactory getPartFactory() {
		return new ILPartFactoryVisitor();
	}

	public ICommandFactory getCommandFactory() {
		return ILCommandFactory.instance();
	}

	public IActionFactory getActionFactory() {
		return ILActionFactory.instance();
	}

	public IExamplesFactory getExamplesFactory() {
		return ILExamplesFactory.instance();
	}

	public boolean hasInterpreter() {
		return true;
	}
	public boolean hasTextGenerator() {
		return true;
	}
	public boolean hasModelGenerator() {
		return true;
	}

	public void interpret(IEntity program, PrintStream out) {
		InterpreterVisitor.interpret((IILEntity)program, out);
	}

	public void generateText(IEntity entity, PrintStream out) {
		PrettyPrinterVisitor.prettyPrint((IILEntity)entity, out);
	}
	
	public IEntity generateModel(IEntity le) {
		return (IEntity)((IILEntity)le).clone();
	}
}
