package unibo.cs.labss.editor.il.actions;

import unibo.cs.labss.editor.AbstractActionFactory;
import unibo.cs.labss.editor.IActionFactory;
import unibo.cs.labss.editor.actions.IEntityTransformer;
import unibo.cs.labss.editor.il.model.Statement;
import unibo.cs.labss.editor.il.model.impl.AbstractBinaryExpression;
import unibo.cs.labss.editor.il.model.impl.IfElseStatement;
import unibo.cs.labss.editor.il.model.impl.IfStatement;
import unibo.cs.labss.editor.il.model.impl.WhileStatement;
import unibo.cs.labss.editor.model.IEntity;

/**
 * @author Riccardo Solmi
 */
public class ILActionFactory extends AbstractActionFactory implements IActionFactory {
	private static IActionFactory instance;
	public static IActionFactory instance() {
		if (instance == null)
			instance = new ILActionFactory();
		return instance;
	}

	protected Object[][] replaceActions() {
		ILEnablerPredicateFactory pf = ILEnablerPredicateFactory.instance;

		final IEntityTransformer fresh = new IEntityTransformer() {
			public void transform(IEntity oldEntity, IEntity newEntity) {
			}
		};
		final IEntityTransformer binOp = new IEntityTransformer() {
			public void transform(IEntity oldEntity, IEntity newEntity) {
				if (oldEntity instanceof AbstractBinaryExpression) {
					((AbstractBinaryExpression)newEntity).setExp1(((AbstractBinaryExpression)oldEntity).getExp1());
					((AbstractBinaryExpression)newEntity).setExp2(((AbstractBinaryExpression)oldEntity).getExp2());
				}
			}
		};

		return new Object[][] {
			{pf.replaceStatement(), "AssignmentStatement", "fresh = ? (assignment)", fresh},
			{pf.replaceStatement(), "BlockStatement", "{?} (block)", new IEntityTransformer() {
				public void transform(IEntity oldEntity, IEntity newEntity) {
					newEntity.addChild(0, (Statement)oldEntity);
				}
			}},
			{pf.replaceStatement(), "IfStatement", "if", new IEntityTransformer() {
				public void transform(IEntity oldEntity, IEntity newEntity) {
					if (oldEntity instanceof IfElseStatement) {
						((IfStatement)newEntity).setExp(((IfElseStatement)oldEntity).getExp());
						((IfStatement)newEntity).setThenStm(((IfElseStatement)oldEntity).getThenStm());
					} else if (oldEntity instanceof WhileStatement) {
						((IfStatement)newEntity).setExp(((WhileStatement)oldEntity).getExp());
						((IfStatement)newEntity).setThenStm(((WhileStatement)oldEntity).getStm());
					}
				}
			}},
			{pf.replaceStatement(), "IfElseStatement", "if else", new IEntityTransformer() {
				public void transform(IEntity oldEntity, IEntity newEntity) {
					if (oldEntity instanceof IfStatement) {
						((IfElseStatement)newEntity).setExp(((IfStatement)oldEntity).getExp());
						((IfElseStatement)newEntity).setThenStm(((IfStatement)oldEntity).getThenStm());
					} else if (oldEntity instanceof WhileStatement) {
						((IfElseStatement)newEntity).setExp(((WhileStatement)oldEntity).getExp());
						((IfElseStatement)newEntity).setThenStm(((WhileStatement)oldEntity).getStm());
					}
				}
			}},
			{pf.replaceStatement(), "WhileStatement", "while", new IEntityTransformer() {
				public void transform(IEntity oldEntity, IEntity newEntity) {
					if (oldEntity instanceof IfStatement) {
						((WhileStatement)newEntity).setExp(((IfStatement)oldEntity).getExp());
						((WhileStatement)newEntity).setStm(((IfStatement)oldEntity).getThenStm());
					} else if (oldEntity instanceof IfElseStatement) {
						((WhileStatement)newEntity).setExp(((IfElseStatement)oldEntity).getExp());
						((WhileStatement)newEntity).setStm(((IfElseStatement)oldEntity).getThenStm());
					}
				}
			}},
			{pf.replaceStatement(), "PrintStatement", "println(?)", fresh},
			{pf.replaceStatement(), "EmptyStatement", "? (statement)", fresh},

			{pf.replaceExpression(), "EmptyExpression", "? (expression)", fresh},

			{pf.replaceIntegerExpression(), "AdditionExpression", "+", binOp},
			{pf.replaceIntegerExpression(), "SubtractionExpression", "-", binOp},
			{pf.replaceIntegerExpression(), "MultiplicationExpression", "*", binOp},
			{pf.replaceIntegerExpression(), "DivisionExpression", "/", binOp},
			{pf.replaceIntegerExpression(), "IntegerLiteral", "0 (literal)", fresh},

			{pf.replaceBooleanExpression(), "AndExpression", "&&", binOp},
			{pf.replaceBooleanExpression(), "OrExpression", "|", binOp},
			{pf.replaceBooleanExpression(), "NotExpression", "!", fresh},
			{pf.replaceBooleanExpression(), "EqualsExpression", "==", binOp},
			{pf.replaceBooleanExpression(), "LessThanExpression", "<", binOp},
			{pf.replaceBooleanExpression(), "GreaterThanExpression", ">", binOp},
			{pf.replaceBooleanExpression(), "BooleanLiteral", "true (literal)", fresh},

			{pf.replaceExpression(), "Variable", "fresh (variable)", fresh}
		};
	}

	protected Object[][] addActions() {
		ILEnablerPredicateFactory pf = ILEnablerPredicateFactory.instance;

		return new Object[][] {
			{pf.insertStatement(), "AssignmentStatement", "fresh = ? (assignment)"},
			{pf.insertStatement(), "BlockStatement", "{?} (block)"},
			{pf.insertStatement(), "IfStatement", "if"},
			{pf.insertStatement(), "IfElseStatement", "if else"},
			{pf.insertStatement(), "WhileStatement", "while"},
			{pf.insertStatement(), "PrintStatement", "println(?)"},
			{pf.insertStatement(), "EmptyStatement", "? (statement)"},
		};
	}
}
