package unibo.cs.labss.editor.il.actions;

import java.util.Iterator;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import unibo.cs.labss.editor.commands.CompositeRemoveCommand;
import unibo.cs.labss.editor.il.model.IILEntity;
import unibo.cs.labss.editor.il.model.Statement;
import unibo.cs.labss.editor.il.model.impl.ILLanguageFactory;
import unibo.cs.labss.editor.model.IEntity;
import unibo.cs.labss.editor.parts.AbstractPart;
import unibo.cs.labss.editor.requests.CommandFactory;
import unibo.cs.labss.editor.requests.DeletePartRequest;
import unibo.cs.labss.editor.requests.DnDOverCompositeRequest;
import unibo.cs.labss.editor.requests.ICommandFactory;
import unibo.cs.labss.editor.requests.IFeatureTransformer;
import unibo.cs.labss.editor.requests.IPartRequestHandler;
import unibo.cs.labss.editor.requests.PartRequest;
import unibo.cs.labss.editor.requests.PartRequestHandler;

public class ILCommandFactory extends CommandFactory {
	private static ILCommandFactory instance;
	public static final ILCommandFactory instance() {
		if (instance == null)
			instance = new ILCommandFactory();
		return instance;
	}

	public ILCommandFactory() {
		final IFeatureTransformer CLONE = new IFeatureTransformer() {
			public IEntity transform(IEntity newFeature) {
				return (IEntity)((IILEntity)newFeature).clone();
			}
		};
		final IFeatureTransformer SHARE = CLONE;

		ILEnablerPredicateFactory pf = ILEnablerPredicateFactory.instance;

		final ICommandFactory removeIndexedFeature = new ICommandFactory() {
			public Command create(PartRequest request) {
				CompositeRemoveCommand cmd = new CompositeRemoveCommand();
				cmd.setComposite(request.getParentModelEntity());
				cmd.setChild(request.getModelEntity());
				if(request.getModelEntity() instanceof Statement)
					cmd.setEmptyReplacement(ILLanguageFactory.instance.createEmptyStatement());
				return cmd;
			}
		};

		final ICommandFactory removeAllIndexedFeatures = new ICommandFactory() {
			public Command create(PartRequest request) {
				CompoundCommand compound = new CompoundCommand();
				Iterator childIterator = request.getPart().getChildren().iterator();
				while(childIterator.hasNext()) {
					AbstractPart part = (AbstractPart) childIterator.next();
					compound.add(removeIndexedFeature.create(new DeletePartRequest(PartRequest.DELETE, part)));
				}
				return compound.unwrap();
			}
		};

		final ICommandFactory insertIndexedFeature = new CompositeAddCommandFactory(IFeatureTransformer.IDENTITY);
		final ICommandFactory insertSharedIndexedFeature = new CompositeAddCommandFactory(SHARE);
		final ICommandFactory insertClonedIndexedFeature = new CompositeAddCommandFactory(CLONE);

		final ICommandFactory orphanIndexedFeature = new ICommandFactory() {
			public Command create(PartRequest request) {
				DnDOverCompositeRequest orderedRequest = (DnDOverCompositeRequest) request;
				return removeIndexedFeature.create(new DeletePartRequest(PartRequest.DELETE,
						orderedRequest.getDnDChild()));
			}
		};

		final ICommandFactory moveIndexedFeature = new ICommandFactory() {
			public Command create(PartRequest request) {
				Command remove = orphanIndexedFeature.create(request);
				Command insert = insertIndexedFeature.create(request);

				CompoundCommand compound = new CompoundCommand();
				compound.add(remove);
				compound.add(insert);
				return compound.unwrap();
			}
		};

		final ICommandFactory replaceFeature = new ReplaceChildCommandFactory(IFeatureTransformer.IDENTITY);
		final ICommandFactory replaceSharedFeature = new ReplaceChildCommandFactory(SHARE);
		final ICommandFactory replaceClonedFeature = new ReplaceChildCommandFactory(CLONE);
		final ICommandFactory replaceWithEmpty = new ReplaceChildCommandFactory(new IFeatureTransformer() {
			public IEntity transform(IEntity newFeature) {
				return ILLanguageFactory.instance.createEmptyExpression();
			}
		});
		final ICommandFactory replaceWithEmptyInteger = new ReplaceChildCommandFactory(new IFeatureTransformer() {
			public IEntity transform(IEntity newFeature) {
				return ILLanguageFactory.instance.createEmptyIntegerExpression();
			}
		});
		final ICommandFactory replaceWithEmptyBoolean = new ReplaceChildCommandFactory(new IFeatureTransformer() {
			public IEntity transform(IEntity newFeature) {
				return ILLanguageFactory.instance.createEmptyBooleanExpression();
			}
		});

		final ICommandFactory compoundReplaceFeature = new CompoundReplaceChildCommandFactory(IFeatureTransformer.IDENTITY);
		final ICommandFactory compoundReplaceSharedFeature = new CompoundReplaceChildCommandFactory(SHARE);
		final ICommandFactory compoundReplaceClonedFeature = new CompoundReplaceChildCommandFactory(CLONE);

		addHandlers(PartRequest.DELETE, new IPartRequestHandler[] {
			new PartRequestHandler(pf.hasCompositeParent(), removeIndexedFeature),
			new PartRequestHandler(pf.isComposite(), removeAllIndexedFeatures),
			new PartRequestHandler(pf.isNonEmptyIntegerExpression(), replaceWithEmptyInteger),
			new PartRequestHandler(pf.isNonEmptyBooleanExpression(), replaceWithEmptyBoolean),
			new PartRequestHandler(pf.isNonEmptyExpression(), replaceWithEmpty),
		});

		addHandlers(PartRequest.MOVE_ORPHAN_CHILD, new IPartRequestHandler[] {
			new PartRequestHandler(pf.isComposite(), orphanIndexedFeature),
		});

		addHandlers(PartRequest.MOVE_ADD_CHILD, new IPartRequestHandler[] {
			new PartRequestHandler(pf.dndStatementOverBlockStatement(), insertIndexedFeature),
			new PartRequestHandler(pf.dndStatementOverEmptyStatementInBlock(), compoundReplaceFeature),
		});

		addHandlers(PartRequest.MOVE_CHILD, new IPartRequestHandler[] {
			new PartRequestHandler(pf.isComposite(), moveIndexedFeature),
		});

		addHandlers(PartRequest.CLONE_CHILD, new IPartRequestHandler[] {
			new PartRequestHandler(pf.dndStatementOverBlockStatement(), insertClonedIndexedFeature),
			new PartRequestHandler(pf.dndStatementOverEmptyStatementInBlock(), compoundReplaceClonedFeature),
			new PartRequestHandler(pf.dndSingleExpressionOverExpression(), replaceClonedFeature),
		});

		addHandlers(PartRequest.SHARE_CHILD, new IPartRequestHandler[] {
			new PartRequestHandler(pf.dndStatementOverBlockStatement(), insertSharedIndexedFeature),
			new PartRequestHandler(pf.dndStatementOverEmptyStatementInBlock(), compoundReplaceSharedFeature),
			new PartRequestHandler(pf.dndSingleExpressionOverExpression(), replaceSharedFeature),
		});
	}
}
