package unibo.cs.labss.editor.il.actions;

import unibo.cs.labss.editor.actions.EnablerPredicateFactory;
import unibo.cs.labss.editor.actions.IEnablerPredicate;
import unibo.cs.labss.editor.il.model.BooleanExpression;
import unibo.cs.labss.editor.il.model.Expression;
import unibo.cs.labss.editor.il.model.IntegerExpression;
import unibo.cs.labss.editor.il.model.LogicalExpression;
import unibo.cs.labss.editor.il.model.Statement;
import unibo.cs.labss.editor.il.model.impl.AssignmentStatement;
import unibo.cs.labss.editor.il.model.impl.BlockStatement;
import unibo.cs.labss.editor.il.model.impl.EmptyBooleanExpression;
import unibo.cs.labss.editor.il.model.impl.EmptyExpression;
import unibo.cs.labss.editor.il.model.impl.EmptyIntegerExpression;
import unibo.cs.labss.editor.il.model.impl.EmptyStatement;
import unibo.cs.labss.editor.il.model.impl.Variable;
import unibo.cs.labss.editor.parts.AbstractPart;

/**
 * @author Riccardo Solmi
 */
public class ILEnablerPredicateFactory extends EnablerPredicateFactory {
	public static final ILEnablerPredicateFactory instance = new ILEnablerPredicateFactory();
	private ILEnablerPredicateFactory() {}

	public final IEnablerPredicate replaceDifferent() {
		return and(replaceMode(), differentType());
	}

	public final IEnablerPredicate replaceStatement() {
		return and(replaceDifferent(), instanceOf(Statement.class));
	}
	public final IEnablerPredicate insertStatement() {
		return and(insertMode(), instanceOf(BlockStatement.class));
	}

	public final IEnablerPredicate replaceExpression() {
		return and(replaceDifferent(), instanceOf(Expression.class));
	}

	public final IEnablerPredicate inAssignmentRvalue() {
		return and(parent_instanceOf(AssignmentStatement.class), new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return ((AssignmentStatement)selectedPart.getParentModelEntity()).getExp() == selectedPart.getModel();
			}
		});
	}
	public final IEnablerPredicate inAssignmentLvalue() {
		return and(parent_instanceOf(AssignmentStatement.class), new IEnablerPredicate() {
			public boolean evaluate(AbstractPart selectedPart, Object userdata) {
				return ((AssignmentStatement)selectedPart.getParentModelEntity()).getId() == selectedPart.getModel();
			}
		});
	}


	public final IEnablerPredicate isLogicalVariable() {
		return and(instanceOf(Variable.class), parent_instanceOf(LogicalExpression.class));
	}
	public final IEnablerPredicate isIntegerVariable() {
		return and(instanceOf(Variable.class), not(parent_instanceOf(LogicalExpression.class)));
	}

	public final IEnablerPredicate replaceIntegerExpression() {
		return and(replaceDifferent(), or(
				instanceOf(IntegerExpression.class),
				inAssignmentRvalue(),
				and(isIntegerVariable(), not(inAssignmentLvalue()))));
	}

	public final IEnablerPredicate replaceBooleanExpression() {
		return and(replaceDifferent(), or(
				instanceOf(BooleanExpression.class),
				inAssignmentRvalue(),
				and(isLogicalVariable(), not(inAssignmentLvalue()))));
	}

	// DnD Enabler Predicates
	public final IEnablerPredicate dndStatementOverBlockStatement() {
		return dndOver(Statement.class, BlockStatement.class);
	}

	public final IEnablerPredicate dndStatementOverEmptyStatementInBlock() {
		return and(dndOver(Statement.class, EmptyStatement.class), hasCompositeParent());
	}

	public final IEnablerPredicate dndSingleExpressionOverExpression() {
		return and(dndSinglePart(), dndOver(Expression.class, Expression.class));
	}

	public final IEnablerPredicate isNonEmptyExpression() {
		return and(instanceOf(Expression.class), not(instanceOf(EmptyExpression.class)));
	}
	public final IEnablerPredicate isNonEmptyIntegerExpression() {
		return and(instanceOf(Expression.class), not(instanceOf(EmptyIntegerExpression.class)));
	}
	public final IEnablerPredicate isNonEmptyBooleanExpression() {
		return and(instanceOf(Expression.class), not(instanceOf(EmptyBooleanExpression.class)));
	}
}
