package unibo.cs.labss.editor.il.model.impl;

import unibo.cs.labss.editor.AbstractExamplesFactory;
import unibo.cs.labss.editor.il.ILEditorKit;
import unibo.cs.labss.editor.il.model.Expression;
import unibo.cs.labss.editor.il.model.IILEntity;
import unibo.cs.labss.editor.il.model.IILFactory;
import unibo.cs.labss.editor.il.model.Statement;
import unibo.cs.labss.editor.il.model.visitors.InterpreterVisitor;
import unibo.cs.labss.editor.il.model.visitors.PrettyPrinterVisitor;
import unibo.cs.labss.editor.model.IEntity;

public class ILExamplesFactory extends AbstractExamplesFactory {
	private static ILExamplesFactory instance = new ILExamplesFactory();
	public static final ILExamplesFactory instance() {
		if (instance == null)
			instance = new ILExamplesFactory();
		return instance;
	}

	public static void main(String[] args) {
		System.out.println("1================");
		PrettyPrinterVisitor.prettyPrint(instance.factProgram(), System.out);
		System.out.println("2================");
		InterpreterVisitor.interpret(instance.factProgram(), System.out);
		System.out.println("3================");
		PrettyPrinterVisitor.prettyPrint((IILEntity)instance.factProgram().clone(), System.out);
	}

	IILFactory lf;
	private ILExamplesFactory() {
		lf = ILLanguageFactory.instance;

		put("empty", new ExampleFactory() {
			public IEntity create() {
				return emptyProgram();
			}
		});
		put("Factorial (shared vars)", new ExampleFactory() {
			public IEntity create() {
				return sharedFactProgram();
			}
		});
		put("Factorial", new ExampleFactory() {
			public IEntity create() {
				return factProgram();
			}
		});
		put("Sample", new ExampleFactory() {
			public IEntity create() {
				return sampleProgram();
			}
		});
	}

	protected String getModelId() {
		return ILEditorKit.ID;
	}

	public IILEntity emptyProgram() {
		return lf.createProgram(lf.createEmptyStatement());
	}

	public IILEntity sharedFactProgram() {
		Expression n = lf.createVariable("n");
		Expression result = lf.createVariable("result");
		Statement fatt = lf.createBlock(new Statement[]{
				lf.createAssignment(n, lf.createLiteral(4)),
				lf.createAssignment(result, lf.createLiteral(1)),
				lf.createWhile(lf.createGreaterThan(n, lf.createLiteral(0)), lf.createBlock(new Statement[]{
						lf.createAssignment(result, lf.createMultiplication(result, n)),
						lf.createAssignment(n, lf.createSubtraction(n, lf.createLiteral(1)))
				})),
				lf.createPrint(result)
		});
		return lf.createProgram(fatt);
	}

	public IILEntity factProgram() {
		return lf.createProgram(lf.createBlock(new Statement[]{
				lf.createAssignment(lf.createVariable("n"), lf.createLiteral(4)),
				lf.createAssignment(lf.createVariable("result"), lf.createLiteral(1)),
				lf.createWhile(lf.createGreaterThan(lf.createVariable("n"), lf.createLiteral(0)), lf.createBlock(new Statement[]{
						lf.createAssignment(lf.createVariable("result"), lf.createMultiplication(lf.createVariable("result"), lf.createVariable("n"))),
						lf.createAssignment(lf.createVariable("n"), lf.createSubtraction(lf.createVariable("n"), lf.createLiteral(1)))
				})),
				lf.createPrint(lf.createVariable("result"))
		}));
	}

	public IILEntity sampleProgram() {
		Statement fatt = lf.createBlock(new Statement[]{
				lf.createAssignment(lf.createVariable("n"), lf.createLiteral(4)),
				lf.createAssignment(lf.createVariable("result"), lf.createLiteral(1)),
				lf.createWhile(lf.createAnd(lf.createGreaterThan(lf.createVariable("n"), lf.createLiteral(0)), lf.createNot(lf.createLiteral(false))), lf.createBlock(new Statement[]{
						lf.createAssignment(lf.createVariable("result"), lf.createMultiplication(lf.createVariable("result"), lf.createVariable("n"))),
						lf.createAssignment(lf.createVariable("n"), lf.createSubtraction(lf.createVariable("n"), lf.createLiteral(1)))
				})),
				lf.createWhile(lf.createNot(lf.createLessThan(lf.createVariable("n"), lf.createLiteral(0))), lf.createBlock(new Statement[]{
						lf.createAssignment(lf.createVariable("result"), lf.createMultiplication(lf.createAddition(lf.createVariable("result"), lf.createVariable("result")), lf.createVariable("n"))),
						lf.createAssignment(lf.createVariable("n"), lf.createSubtraction(lf.createVariable("n"), lf.createLiteral(1)))
				})),
				lf.createIf(lf.createNot(lf.createLiteral(true)), lf.createAssignment(lf.createVariable("result"), lf.createMultiplication(lf.createAddition(lf.createVariable("result"), lf.createVariable("result")), lf.createVariable("n"))))
		});
		return lf.createProgram(fatt);
	}
}
