package unibo.cs.labss.editor.il.model.impl;

import unibo.cs.labss.editor.il.model.Expression;
import unibo.cs.labss.editor.il.model.IILEntity;
import unibo.cs.labss.editor.il.model.IILFactory;
import unibo.cs.labss.editor.il.model.Literal;
import unibo.cs.labss.editor.il.model.Statement;


/**
 * @author Riccardo Solmi
 */
public class ILLanguageFactory implements IILFactory {
	public static final IILFactory instance = new ILLanguageFactory();
	private ILLanguageFactory() {
	}
	
	public IILEntity createProgram(IILEntity rootEntity) {
		return new Program(rootEntity);
	}

	public Statement createWhile(Expression exp, Statement stm) {
		return new WhileStatement(exp, stm);
	}
	
	public Statement createIf(Expression exp, Statement thenStm) {
		return new IfStatement(exp, thenStm);
	}
	
	public Statement createIf(Expression exp, Statement thenStm, Statement elseStm) {
		return new IfElseStatement(exp, thenStm, elseStm);
	}
	
	public Statement createAssignment(Expression var, Expression exp) {
		return new AssignmentStatement(var, exp);
	}
	
	public Statement createBlock(Statement[] stm) {
		return new BlockStatement(stm);
	}

	public Statement createPrint(Expression var) {
		return new PrintStatement(var);
	}
	
	public Literal createLiteral(boolean val) {
		return BooleanLiteral.create(val);
	}
	
	public Literal createLiteral(int val) {
		return new IntegerLiteral(val);
	}
	
	public Expression createVariable(String name) {
		return new Variable(name);
	}
	
	public Expression createAddition(Expression exp1, Expression exp2) {
		return new AdditionExpression(exp1, exp2);
	}
	
	public Expression createSubtraction(Expression exp1, Expression exp2) {
		return new SubtractionExpression(exp1, exp2);
	}
	
	public Expression createMultiplication(Expression exp1, Expression exp2) {
		return new MultiplicationExpression(exp1, exp2);
	}
	
	public Expression createDivision(Expression exp1, Expression exp2) {
		return new DivisionExpression(exp1, exp2);
	}
	
	public Expression createAnd(Expression exp1, Expression exp2) {
		return new AndExpression(exp1, exp2);
	}
	
	public Expression createOr(Expression exp1, Expression exp2) {
		return new OrExpression(exp1, exp2);
	}
	
	public Expression createNot(Expression exp) {
		return new NotExpression(exp);
	}
	
	public Expression createEquals(Expression exp1, Expression exp2) {
		return new EqualsExpression(exp1, exp2);
	}
	
	public Expression createLessThan(Expression exp1, Expression exp2) {
		return new LessThanExpression(exp1, exp2);
	}
	
	public Expression createGreaterThan(Expression exp1, Expression exp2) {
		return new GreaterThanExpression(exp1, exp2);
	}
	
	public Expression createEmptyExpression() {
		return new EmptyExpression();
	}
	public Expression createEmptyBooleanExpression() {
		return new EmptyBooleanExpression();
	}
	public Expression createEmptyIntegerExpression() {
		return new EmptyIntegerExpression();
	}
	
	public Statement createEmptyStatement() {
		return new EmptyStatement();
	}
}