package unibo.cs.labss.editor.il.parts;

import unibo.cs.labss.editor.IPartFactory;
import unibo.cs.labss.editor.il.model.IILEntity;
import unibo.cs.labss.editor.il.model.IILVisitor;
import unibo.cs.labss.editor.il.model.Literal;
import unibo.cs.labss.editor.il.model.impl.AdditionExpression;
import unibo.cs.labss.editor.il.model.impl.AndExpression;
import unibo.cs.labss.editor.il.model.impl.AssignmentStatement;
import unibo.cs.labss.editor.il.model.impl.BlockStatement;
import unibo.cs.labss.editor.il.model.impl.DivisionExpression;
import unibo.cs.labss.editor.il.model.impl.EmptyBooleanExpression;
import unibo.cs.labss.editor.il.model.impl.EmptyExpression;
import unibo.cs.labss.editor.il.model.impl.EmptyIntegerExpression;
import unibo.cs.labss.editor.il.model.impl.EmptyStatement;
import unibo.cs.labss.editor.il.model.impl.EqualsExpression;
import unibo.cs.labss.editor.il.model.impl.GreaterThanExpression;
import unibo.cs.labss.editor.il.model.impl.IfElseStatement;
import unibo.cs.labss.editor.il.model.impl.IfStatement;
import unibo.cs.labss.editor.il.model.impl.LessThanExpression;
import unibo.cs.labss.editor.il.model.impl.MultiplicationExpression;
import unibo.cs.labss.editor.il.model.impl.NotExpression;
import unibo.cs.labss.editor.il.model.impl.OrExpression;
import unibo.cs.labss.editor.il.model.impl.PrintStatement;
import unibo.cs.labss.editor.il.model.impl.Program;
import unibo.cs.labss.editor.il.model.impl.SubtractionExpression;
import unibo.cs.labss.editor.il.model.impl.Variable;
import unibo.cs.labss.editor.il.model.impl.WhileStatement;
import unibo.cs.labss.editor.model.IEntity;
import unibo.cs.labss.editor.parts.AbstractPart;
import unibo.cs.labss.editor.parts.CompositeColumnPart;
import unibo.cs.labss.editor.parts.EmptyPart;

/**
 * @author Riccardo Solmi
 */
public class ILPartFactoryVisitor implements IPartFactory, IILVisitor {
    AbstractPart part = null;

    public synchronized AbstractPart createPart(IEntity modelEntity) {
    	part = null;
    	((IILEntity)modelEntity).accept(this);
        return part;
    }

	public void visitAddition(AdditionExpression exp) {
		part = new BinaryExpressionPart("+");
	}
	public void visitAnd(AndExpression exp) {
		part = new BinaryExpressionPart("&&");
	}
	public void visitAssignment(AssignmentStatement stm) {
		part = new AssignmentStatementPart();
	}
	public void visitBlock(BlockStatement stm) {
		part = new CompositeColumnPart();
	}
	public void visitBooleanLiteral(Literal exp) {
		part = new LiteralPart();
	}
	public void visitDivision(DivisionExpression exp) {
		part = new BinaryExpressionPart("/");
	}
	public void visitEmpty(EmptyExpression exp) {
    	part = new EmptyPart();
	}
	public void visitEmpty(EmptyStatement exp) {
       	part = new EmptyPart();
	}
	public void visitEmptyBoolean(EmptyBooleanExpression exp) {
    	part = new EmptyPart();
	}
	public void visitEmptyInteger(EmptyIntegerExpression exp) {
    	part = new EmptyPart();
	}
	public void visitEquals(EqualsExpression exp) {
    	part = new BinaryExpressionPart("==");
	}
	public void visitGreaterThan(GreaterThanExpression exp) {
    	part = new BinaryExpressionPart(">");
	}
	public void visitVariable(Variable exp) {
    	part = new VariablePart();
	}
	public void visitIf(IfStatement stm) {
    	part = new IfStatementPart();
	}
	public void visitIfElse(IfElseStatement stm) {
    	part = new IfElseStatementPart();
	}
	public void visitIntegerLiteral(Literal exp) {
		part = new LiteralPart();
	}
	public void visitLessThan(LessThanExpression exp) {
    	part = new BinaryExpressionPart("<");
	}
	public void visitMultiplication(MultiplicationExpression exp) {
    	part = new BinaryExpressionPart("*");
	}
	public void visitNot(NotExpression exp) {
       	part = new NotExpressionPart();
	}
	public void visitOr(OrExpression exp) {
    	part = new BinaryExpressionPart("||");
	}
	public void visitPrint(PrintStatement stm) {
    	part = new PrintStatementPart();
	}
	public void visitProgram(Program p) {
    	part = new ProgramPart();
	}
	public void visitSubtraction(SubtractionExpression exp) {
    	part = new BinaryExpressionPart("-");
	}
	public void visitVoidLiteral(Literal exp) {
		part = new LiteralPart();
	}
	public void visitWhile(WhileStatement stm) {
    	part = new WhileStatementPart();
	}
}
